/*****************************************************************************
 *          Copyright: (C) Copyright David Goodwin, 2014
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

/*
 * database schema compatibility checks.
 */

#include "dbutil.h"
#include "constants.h"

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QVariant>
#include <QtDebug>

using namespace DbUtil;


int DbUtil::getDatabaseVersion(QSqlDatabase db) {
    QSqlQuery query("select * "
                    "from information_schema.tables "
                    "where table_schema = 'public' "
                    "  and table_name = 'db_info'", db);
    if (!query.isActive()) {
        return -1;
    } else if (query.size() == 1){
        // It is at least a v2 (zxweather 0.2) schema.
        query.exec("select v::integer "
                   "from DB_INFO "
                   "where k = 'DB_VERSION'");
        if (!query.isActive() || query.size() != 1) {
            return -1;
        } else {
            query.first();
            return query.value(0).toInt();
        }
    }

    return 1;
}

DatabaseCompatibility DbUtil::checkDatabaseCompatibility(QSqlDatabase db, bool reading) {

    auto dbType = getDatabaseType(db);
    int version = getDatabaseVersion(db);

    if (!reading && (dbType == DT_ZXW_V1 || dbType == DT_ZXW_V2)) {
        /*
         * Logging to zxweather databases is entirely unsupported.
         */
        return DbUtil::DC_Incompatible;
    }

    if (dbType == DT_TEMPVIEW && version == 1) {
        /*
         * The TempView v1.x schema (level 1) is not forwards compatible with
         * TempView 2.x. It must be upgraded before it can be used. The upgraded
         * database will still be backwards compatible with TempView 1.x
         */
        return DbUtil::DC_Incompatible;
    }

    // So we're either writing to a TempView schema level 2 database, or we're
    // reading from an older (or zxweather) database.

    qDebug() << "Schema version:" << version;
    if (version == -1) {
        return DbUtil::DC_BadSchemaVersion;
    } else if (version == 1) {
        /*
         * zxweather v0.x or TempView v1.x. We're OK to read from this.
         */
        qDebug() << "V1 database";
        return DbUtil::DC_Compatible;
    } else if (version > 1) {
        qDebug() << "zxweather schema 2+ database or tempview v2.0+ database";

        // Check that this version of tempview hasn't been blacklisted
        // by the database.
        QSqlQuery query("select version_check('tempview'," +
                        Constants::DB_VERSION_CHECK_PARAMS + ")", db);
        if (!query.isActive())
            return DbUtil::DC_Unknown;
        else {
            query.first();
            if (!query.value(0).toBool()) {
                return DbUtil::DC_Incompatible_Version;
            }
        }
    }
    return DbUtil::DC_Compatible;
}

DatabaseType DbUtil::getDatabaseType(QSqlDatabase db) {
    int version = getDatabaseVersion(db);

    if (version == -1) // Getting schema version failed.
        return DbUtil::DT_UNKNOWN;

    if (version == 1) {
        /*
         * TempView v1 or zxweather v1. If its zxweather then the
         * db_info table will be missing (zxweather didn't have this
         * table until schema v2, zxweather 1.0+)
         */
        QSqlQuery query("select * "
                        "from information_schema.tables "
                        "where table_schema = 'public' "
                        "  and table_name = 'db_info'", db);
        if (!query.isActive()) {
            return DbUtil::DT_UNKNOWN; // Query failed.
        } else if (query.size() == 1){
            /*
             * TempView. zxweather v0.x (v1 schema) doesn't have this
             * table.
             */
            return DbUtil::DT_TEMPVIEW;
        }
    } else {
        /*
         * zxweather v2 schema or some other version of TempView. The
         * db_info table will tell us which.
         */

        QSqlQuery query("select v::boolean "
                   "from DB_INFO "
                   "where k = 'TEMPVIEW'");
        if (!query.isActive())
            return DbUtil::DT_UNKNOWN; // Query failed.
        else if (query.size() != 1) {
            /*
             * The TEMPVIEW key won't be present in zxweather.
             */
            return DbUtil::DT_ZXW_V2;
        } else {
            query.first();
            if (query.value(0).toBool()) {
                /*
                 * TEMPVIEW key is present and set. We're looking at a TempView
                 * database.
                 */
                return DbUtil::DT_TEMPVIEW;
            } else {
                /*
                 * db_info table is there, TEMPVIEW key is present
                 * but its not set. Probably zxweather or something
                 * compatible with it.
                 */
                return DbUtil::DT_ZXW_V2;
            }
        }

    }

    return DbUtil::DT_UNKNOWN;
}

QString DbUtil::getMinimumAppVersion(QSqlDatabase db)
{
    QSqlQuery query("select minimum_version_string('tempview')", db);

    QString result;

    if (query.isActive()) {
        query.first();
        result = query.value(0).toString();
    }

    return result;
}
