/*****************************************************************************
 *          Copyright: (C) Copyright David Goodwin, 2014
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

#ifndef DBUTIL_H
#define DBUTIL_H

#include <QString>
#include <QSqlDatabase>
#include <QDateTime>
#include <QList>
#include <QMetaType>

/** Functions for checking compatibility of the database.
 * Details with both TempView and zxweather database schemas.
 *
 */
namespace DbUtil {

/** Result from a database compatibility check.
 */
typedef enum {
    DC_Compatible, /*!< Database is fully compatible */
    DC_Incompatible, /*!< Database is incompatible */
    DC_Incompatible_Version, /*!< Database needs a newer version of the application */
    DC_BadSchemaVersion, /*!< Failed to determine database schema version. Incompatible. */
    DC_Unknown /*!< Could not determine compatibility. Might work. Might not. */
} DatabaseCompatibility;

/** Checks to see if the specified database is compatible with this version
 * of the zxweather desktop client.
 *
 * @param db Database to check compatibility of.
 * @param reading If we're checking compatibility for reading or writing.
 * @return Result of the database compatibility check.
 */
DatabaseCompatibility checkDatabaseCompatibility(QSqlDatabase db, bool reading);

/** Gets the minimum version of the desktop client required to connect to
 * the specified database. The minimum version is returned as a human-readable
 * string (eg, 1.5.2)
 *
 * @param db Database to get the minimum app version for.
 * @return The minimum version of the app required.
 */
QString getMinimumAppVersion(QSqlDatabase db);

/** Gets the database schema version. Used to check if the database is new
 * enough for this version of the desktop client.
 *
 * @param db Database to get the schema version of.
 * @return Database schema version.
 */
int getDatabaseVersion(QSqlDatabase db);

typedef enum {
    DT_UNKNOWN,
    DT_ZXW_V1,
    DT_ZXW_V2,
    DT_TEMPVIEW
} DatabaseType;

/** Tries to figure out what sort of database we're connected to.
 * It could be zxweather schema v1, zxweather schema v2+, TempView
 * or unknown.
 *
 * Logging to zxweather databases is not supported (as this program
 * only supports a single temperature sensor - no humidity, rainfall,
 * etc).
 *
 * Using a zxweather v1 database as a datasource works fine - you'll
 * get the outside temperature. Using a v2 database requires
 * configuration file adjustments (to specify the station code).
 *
 * @param db Database to check
 * @return true if the database is zxweather schema v2+
 */
DatabaseType getDatabaseType(QSqlDatabase db);

}

#endif // DBUTIL_H
