/*****************************************************************************
 *          Copyright: (C) Copyright David Goodwin, 2014
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

#include "postgresqldatalogger.h"

#include "settings.h"

#include "dbutil.h"

#include <QSqlDatabase>
#include <QStringList>
#include <QSqlQuery>
#include <QSqlError>
#include <QtDebug>
#include <QSqlRecord>

PostgreSQLDataLogger::PostgreSQLDataLogger(QObject *parent) :
    AbstractDataLogger(parent)
{
    isConnected = false;
}

PostgreSQLDataLogger::~PostgreSQLDataLogger() {
    disconnectLogger();
}

bool PostgreSQLDataLogger::connectLogger() {

    Settings& settings = Settings::getInstance();

    if (QSqlDatabase::isDriverAvailable("QPSQL")) {

        // Ensure we're not connected to anything else
        if (QSqlDatabase::database(QSqlDatabase::defaultConnection, false).isValid()) {
            qWarning() << "Database is already open (SHOULD NOT HAPPEN!)";
            auto db = QSqlDatabase::database()   ;
            db.close();
            QSqlDatabase::removeDatabase(QSqlDatabase::defaultConnection);
        }

        QSqlDatabase db = QSqlDatabase::addDatabase("QPSQL");
        db.setHostName(settings.databaseHostName());
        db.setPort(settings.databasePort());
        db.setDatabaseName(settings.databaseName());
        db.setUserName(settings.databaseUsername());
        db.setPassword(settings.databasePassword());

        bool ok = db.open();
        if (!ok) {
            emit error(tr("Database Connect Failed"),
                       QString(tr("Failed to connect to the database. Check database "
                       "settings. The error was: %1")).arg(
                           db.lastError().driverText()));
            return false;
        }

        auto compatibility = DbUtil::checkDatabaseCompatibility(db, false);

        if (compatibility == DbUtil::DC_Incompatible_Version) {
            QString ver = DbUtil::getMinimumAppVersion(db);
            emit error(tr("Incompatbile Database"),
                       QString(tr("The configured database is not compatbile with this"
                       " of version TempView.\nPlease upgrade to TempView version %1 or"
                       " newer.")).arg(ver));
            return false;
        }
        else if (compatibility != DbUtil::DC_Compatible) {
            emit error(tr("Incompatible Database"),
                       tr("The configured database is incompatible with this application."
                       " Check database configuration."));
            return false; // Incompatible database.
        }

        // Do other normal initialisation stuff.
        if (!AbstractDataLogger::connectLogger())
            return false;

        // Done.
        isConnected = true;
        return true;

    } else {
        emit error(tr("Database Driver Not Found"),
                   tr("The Qt PostgreSQL database driver was not found."
                   " Unable to use remote PostgreSQL database as a"
                   " data source."));
        return false;
    }
}

void PostgreSQLDataLogger::insertSample(qint64 timestamp, float temperature,
                                        float humidity, float dewPoint) {
    if (isConnected) {
        QSqlQuery qry;
        if (humidity > 0) {
            qry.prepare("insert into sample(time_stamp, temperature, humidity, "
                        "dew_point) values(:ts, :temp, :hum, :dp)");
            qry.bindValue(":ts", QDateTime::fromMSecsSinceEpoch(timestamp));
            qry.bindValue(":temp", temperature);
            qry.bindValue(":hum", humidity);
            qry.bindValue(":dp", dewPoint);
        } else {
            qry.prepare("insert into sample(time_stamp, temperature) "
                        "values(:ts, :temp)");
            qry.bindValue(":ts", QDateTime::fromMSecsSinceEpoch(timestamp));
            qry.bindValue(":temp", temperature);
        }
        if (!qry.exec()) {
            qWarning() << "Failed to log data";
            qDebug() << qry.lastError().databaseText();
            qDebug() << qry.lastError().driverText();
        }
    }
}

void PostgreSQLDataLogger::logData(float temperature, float humidity,
                                   float dewPoint) {

    // Do normal sample preparation stuff.
    AbstractDataLogger::logData(temperature, humidity, dewPoint);

    // Lastly, broadcast temperature to any other clients on the database.
    if (isConnected) {
        QString command = "notify live_data_updated_v2, '%1,%2'";
        QString queryText;
        if (humidity > 0) {
            queryText = command.arg(temperature).arg(humidity);
        } else {
            queryText = command.arg(temperature).arg("-").arg("-");
        }
        QSqlQuery qry;
        if (!qry.exec(queryText)) {
            qWarning() << "Failed to broadcast live data";
            qDebug() << qry.lastError().databaseText();
            qDebug() << qry.lastError().driverText();
        }

        // For TempView 1.x compatibility
        if (!qry.exec("notify live_data_updated, '" +
                      QString::number(temperature) + "'" )) {
            qWarning() << "Failed to broadcast live data";
            qDebug() << qry.lastError().databaseText();
            qDebug() << qry.lastError().driverText();
        }
    }
}

void PostgreSQLDataLogger::disconnectLogger(){
    isConnected = false;
    auto db = QSqlDatabase::database();
    db.close();
    QSqlDatabase::removeDatabase(QSqlDatabase::defaultConnection);
}

SampleSet PostgreSQLDataLogger::fetchSamples(SampleColumns columns,
                                             QDateTime start, QDateTime end) {
    SampleSet s;

    QString columnList = "select time_stamp ";
    if (columns.testFlag(SC_Temperature)) columnList += ", temperature ";
    if (columns.testFlag(SC_Humidity))    columnList += ", humidity ";
    if (columns.testFlag(SC_DewPoint))   columnList += ", dew_point ";

    QString from = " from sample "
                   "where time_stamp >= :startTs "
                   "  and time_stamp <= :endTs "
                   "order by time_stamp";

    QString queryText = columnList + from;

    QSqlQuery query;
    query.prepare(queryText);
    query.bindValue(":startTs", start);
    query.bindValue(":endTs", end);

    bool result = query.exec();

    if (!result || !query.isActive()) {
        emit error(tr("Database Error"), query.lastError().driverText());
        return s;
    }

    while (query.next()) {
        QSqlRecord record = query.record();

        time_t timestamp = record.value("time_stamp").toDateTime().toTime_t();
        s.timestamp.append(timestamp);
        s.timestampUnix.append(timestamp);

        if (columns.testFlag(SC_Temperature))
            s.temperature.append(record.value("temperature").toDouble());
        if (columns.testFlag(SC_Humidity))
            s.humidity.append(record.value("humidity").toDouble());
        if (columns.testFlag(SC_DewPoint))
            s.dew_point.append(record.value("dew_point").toDouble());
    }

    return s;
}
