/*****************************************************************************
 *          Copyright: (C) Copyright David Goodwin, 2014
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

#include "settings.h"

#include <QCoreApplication>
#include <QFile>
#include <QVariant>

/** Settings keys.
 */
namespace SettingsKey {
    /** General settings (minimise to system tray, etc).
     */
    namespace General {
        const QString MINIMISE_TO_SYSTRAY = "General/minimise_to_systray";
        const QString CLOSE_TO_SYSTRAY = "General/close_to_systray";
        const QString LOG_TO_DB = "General/log_to_db";
        const QString PROMPT_ON_EXIT_IF_LOGGING = "General/prompt_on_exit_if_logging";
    }

    /** Settings about where to get data from.
     */
    namespace DataSource {
        const QString LIVE_TYPE = "DataSource/type";
        const QString ZXW_STATION_CODE = "DataSource/stationcode";
        const QString ZXW_INDOOR_SENSORS = "DataSource/use_indoor_sensors";
        const QString TEMP_CALIBRATION_OFFSET = "DataSource/calibration_offset";
        const QString HUM_CALIBRATION_OFFSET = "DataSource/humidity_calibration_offset";
    }

    namespace Database {
        const QString DB_DRIVER = "Database/driver";
        const QString FILENAME = "Database/filename";
        const QString NAME = "Database/name";
        const QString HOST_NAME = "Database/hostname";
        const QString PORT = "Database/port";
        const QString USERNAME = "Database/username";
        const QString PASSWORD = "Database/password";
    }

    namespace Colours {
        namespace Charts {
            const QString TEMPERATURE = "Colours/Charts/temperature";
            const QString HUMIDITY = "Colours/Charts/humidity";
            const QString DEW_POINT = "Colours/Charts/dew_point";
        }
    }

    namespace Charts {
        namespace MainWindowGraphs {
            const QString TEMPERATURE = "Charts/MainWindowGraphs/temperature";
            const QString HUMIDITY = "Charts/MainWindowGraphs/humidity";
            const QString DEW_POINT = "Charts/MainWindowGraphs/dew_point";
            const QString LEGEND = "Charts/MainWindowGraphs/legend";
            const QString AVERAGED = "Charts/MainWindowGraphs/averaged";
        }
    }

    /** For tracking what single-shot events have happened (for example,
     * showing a message to the user the first time the program minimises
     * to the system tray.
     */
    namespace SingleShot {
        const QString MINIMISE_TO_SYS_TRAY_INFO = "SingleShot/minimise_to_systray_info";
        const QString CLOSE_TO_SYS_TRAY_INFO = "SingleShot/close_to_systray_info";
        const QString FIRST_RUN = "SingleShot/first_run";
    }
}

Settings::Settings() {
    QString settingsFile;
    settingsFile = QCoreApplication::applicationName() + ".ini";

    if (QFile::exists(settingsFile)) {
        // Load settings from there
        settings = new QSettings(settingsFile, QSettings::IniFormat, this);
    } else {
        settings = new QSettings("zxnet", "tempview", this);
    }
}

Settings::~Settings() {
    delete settings;
    settings = NULL;
}


/* General Settings */
void Settings::setMinimiseToSysTray(bool enabled) {
    settings->setValue(SettingsKey::General::MINIMISE_TO_SYSTRAY,
                       enabled);
}

bool Settings::miniseToSysTray() {
    return settings->value(SettingsKey::General::MINIMISE_TO_SYSTRAY,
                           true).toBool();
}

void Settings::setCloseToSysTray(bool enabled) {
    settings->setValue(SettingsKey::General::CLOSE_TO_SYSTRAY,
                       enabled);
}

bool Settings::closeToSysTray() {
    return settings->value(SettingsKey::General::CLOSE_TO_SYSTRAY,
                           true).toBool();
}

void Settings::setChartColours(ChartColours colours) {
    settings->setValue(SettingsKey::Colours::Charts::TEMPERATURE,
                       colours.temperature);
    settings->setValue(SettingsKey::Colours::Charts::HUMIDITY,
                       colours.humidity);
    settings->setValue(SettingsKey::Colours::Charts::DEW_POINT,
                       colours.dewPoint);
}

ChartColours Settings::getChartColours() {
    ChartColours colours;
    colours.temperature = settings->value(
                SettingsKey::Colours::Charts::TEMPERATURE,
                QColor(Qt::blue)).value<QColor>();

    colours.humidity = settings->value(
                SettingsKey::Colours::Charts::HUMIDITY,
                QColor(Qt::darkMagenta)).value<QColor>();

    colours.dewPoint = settings->value(
                SettingsKey::Colours::Charts::DEW_POINT,
                QColor(Qt::darkCyan)).value<QColor>();

    return colours;
}

void Settings::setMainWindowGraphsEnabled(ChartGraphs graphs) {
     settings->setValue(SettingsKey::Charts::MainWindowGraphs::TEMPERATURE,
                       graphs.temperature);

     settings->setValue(SettingsKey::Charts::MainWindowGraphs::HUMIDITY,
                        graphs.humidity);

     settings->setValue(SettingsKey::Charts::MainWindowGraphs::DEW_POINT,
                        graphs.dewPoint);
}

ChartGraphs Settings::getMainWindowGraphsEnabled() {
    ChartGraphs graphs;
    graphs.temperature = settings->value(
                SettingsKey::Charts::MainWindowGraphs::TEMPERATURE,
                true).toBool();

    graphs.humidity = settings->value(
                SettingsKey::Charts::MainWindowGraphs::HUMIDITY,
                true).toBool();

    graphs.dewPoint = settings->value(
                SettingsKey::Charts::MainWindowGraphs::DEW_POINT,
                false).toBool();

    return graphs;
}

void Settings::setMainWindowGraphLegendEnabled(bool enabled) {
    settings->setValue(SettingsKey::Charts::MainWindowGraphs::LEGEND,
                       enabled);
}

bool Settings::mainWindowGraphLegendEnabled() {
    return settings->value(SettingsKey::Charts::MainWindowGraphs::LEGEND,
                           false).toBool();
}

void Settings::setMainWindowGraphAveraged(bool enabled) {
    settings->setValue(SettingsKey::Charts::MainWindowGraphs::AVERAGED,
                       enabled);
}

bool Settings::mainWindowGraphAveraged() {
    return settings->value(SettingsKey::Charts::MainWindowGraphs::AVERAGED,
                           false).toBool();
}

bool Settings::promptOnExitIfLogging() {
    return settings->value(SettingsKey::General::PROMPT_ON_EXIT_IF_LOGGING, true).toBool();
}

bool Settings::logToDatabase() {
    return settings->value(SettingsKey::General::LOG_TO_DB,
                           false).toBool();
}

void Settings::setLogToDatabase(bool enabled) {
    settings->setValue(SettingsKey::General::LOG_TO_DB,
                       enabled);
}

void Settings::setTemperatureCalibrationOffset(float value) {
    settings->setValue(SettingsKey::DataSource::TEMP_CALIBRATION_OFFSET,
                       value);
}

float Settings::temperatureCalibrationOffset() {
    return settings->value(SettingsKey::DataSource::TEMP_CALIBRATION_OFFSET, 0.0).toFloat();
}

void Settings::setHumidityCalibrationOffset(float value) {
    settings->setValue(SettingsKey::DataSource::HUM_CALIBRATION_OFFSET,
                       value);
}

float Settings::humidityCalibrationOffset() {
    return settings->value(SettingsKey::DataSource::HUM_CALIBRATION_OFFSET, 0.0).toFloat();
}

/* Database Settings */
void Settings::setDatabaseDriver(QString driver) {
    settings->setValue(SettingsKey::Database::DB_DRIVER,
                       driver);
}

QString Settings::databaseDriver() {
    QVariant val = settings->value(SettingsKey::Database::DB_DRIVER, "QPSQL");
    return val.toString();
}

void Settings::setDatabaseFileName(QString filename) {
    settings->setValue(SettingsKey::Database::FILENAME,
                       filename);
}

QString Settings::databaseFileName() {
    QVariant val = settings->value(SettingsKey::Database::FILENAME);
    return val.toString();
}

void Settings::setDatabaseName(QString dbName) {
    settings->setValue(SettingsKey::Database::NAME,
                       dbName);
}

QString Settings::databaseName() {
    QVariant val = settings->value(SettingsKey::Database::NAME);
    return val.toString();
}

void Settings::setDatabaseHostname(QString hostName) {
    settings->setValue(SettingsKey::Database::HOST_NAME,
                       hostName);
}

QString Settings::databaseHostName() {
    QVariant val = settings->value(SettingsKey::Database::HOST_NAME);

    return val.toString();
}

void Settings::setDatabasePort(int port) {
    settings->setValue(SettingsKey::Database::PORT,
                       port);
}

int Settings::databasePort() {
    QVariant val = settings->value(SettingsKey::Database::PORT, 5432);

    return val.toInt();
}

void Settings::setDatabaseUsername(QString username) {
    settings->setValue(SettingsKey::Database::USERNAME,
                       username);
}

QString Settings::databaseUsername() {
    QVariant val = settings->value(SettingsKey::Database::USERNAME);

    return val.toString();
}

void Settings::setDatabasePassword(QString password) {
    settings->setValue(SettingsKey::Database::PASSWORD,
                       password);
}

QString Settings::databasePassword() {
    QVariant val = settings->value(SettingsKey::Database::PASSWORD);

    return val.toString();
}

/* Data Source */
void Settings::setStationCode(QString code) {
    settings->setValue(SettingsKey::DataSource::ZXW_STATION_CODE,
                       code);
}

QString Settings::stationCode() {
    QVariant val = settings->value(SettingsKey::DataSource::ZXW_STATION_CODE);
    return val.toString();
}

void Settings::setUseIndoorSensors(bool enabled) {
    settings->setValue(SettingsKey::DataSource::ZXW_INDOOR_SENSORS,
                       enabled);
}

bool Settings::useIndoorSensors() {
    return settings->value(SettingsKey::DataSource::ZXW_INDOOR_SENSORS,
                           false).toBool();
}

void Settings::setLiveDataSourceType(Settings::data_source_type_t type) {
    int val = 0;

    switch(type) {
    case DS_TYPE_TEMPER:
        val = 0;
        break;
    case DS_TYPE_TEMPERHUM:
        val = 2;
        break;
    case DS_TYPE_DATABASE:
    default:
        val = 1;
    }

    settings->setValue(SettingsKey::DataSource::LIVE_TYPE, val);
}

Settings::data_source_type_t Settings::liveDataSourceType() {

    int val = settings->value(SettingsKey::DataSource::LIVE_TYPE,
                              Settings::DS_TYPE_DATABASE).toInt();

    switch (val) {
    case 1:
        return Settings::DS_TYPE_DATABASE;
        break;
    case 2:
        return Settings::DS_TYPE_TEMPERHUM;
        break;
    case 0:
    default:
        return Settings::DS_TYPE_TEMPER;
        break;
    }
}


/* Single Shot stuff */
void Settings::setSingleShotMinimiseToSysTray() {
    settings->setValue(SettingsKey::SingleShot::MINIMISE_TO_SYS_TRAY_INFO, true);
}

bool Settings::singleShotMinimiseToSysTray() {
    return settings->value(SettingsKey::SingleShot::MINIMISE_TO_SYS_TRAY_INFO,
                           false).toBool();
}

void Settings::setSingleShotCloseToSysTray() {
    settings->setValue(SettingsKey::SingleShot::CLOSE_TO_SYS_TRAY_INFO, true);
}

bool Settings::singleShotCloseToSysTray() {
    return settings->value(SettingsKey::SingleShot::CLOSE_TO_SYS_TRAY_INFO,
                           false).toBool();
}

bool Settings::isFirstRun() {
    bool result = settings->value(SettingsKey::SingleShot::FIRST_RUN,
                                  false).toBool();

    settings->setValue(SettingsKey::SingleShot::FIRST_RUN, true);

    return !result;
}
